__title__ = 'fileblock'
__description__ = 'simplify file processing'
__version__ = '2.0.2.2022.6.13'
__url__ = "http://github.com/miaobuao/fileblock"
__author__ = 'miaobuao'
__author_email__ = '617802189@qq.com'
__license__ = 'Apache 2.0'
__copyright__ = 'Copyright 2022 miaobuao'